# Generated by Django 3.1.5 on 2021-08-12 11:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('backend', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Activity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('topics', models.ManyToManyField(to='backend.Topic')),
            ],
        ),
    ]




class ClientEvent:
    def __init__(self, action, parameters):
        self.action = action
        self.parameters = parameters

    def __iter__(self):
        yield 'action', self.action
        yield 'parameters', self.parameters



from watchdog.observers import Observer


class FileWatcher(object):
    def __init__(self):
        self.handlers = set()
        self.observer = Observer()

    def _schedule_all(self):
        for handler in self.handlers:
            for path in handler.watched_paths:
                self.observer.schedule(handler, path, recursive=True)

    def add_handler(self, handler):
        self.handlers.add(handler)

    def start(self):
        self._schedule_all()
        self.observer.start()

    def stop(self):
        self.observer.stop()



import uuid
import tornado.testing
import tornado.ioloop
from livesync.asyncserver.server import LiveSyncSocketHandler, LiveSyncSocketServer
from livesync.asyncserver.handler import Hub
from livesync.asyncserver.dispatcher import dispatch


class AsyncServerTestCase(tornado.testing.AsyncTestCase):
    def connect(self, session_id=None, client_id=None):
        base_url = 'ws://localhost:{}/?'.format(self.port)
        if session_id:
            base_url += '&session_id={}'.format(session_id)
        if client_id:
            base_url += '&client_id={}'.format(client_id)
        return tornado.websocket.websocket_connect(base_url)

    def setUp(self):
        super(AsyncServerTestCase, self).setUp()
        self.app = LiveSyncSocketServer()
        server = tornado.httpserver.HTTPServer(self.app)
        socket, self.port = tornado.testing.bind_unused_port()
        server.add_socket(socket)

    def tearDown(self):
        tornado.ioloop.IOLoop.instance().stop()

    @tornado.testing.gen_test
    def test_client_receives_welcome_message(self):
        # act
        client = yield self.connect()
        response = yield client.read_message()
        # assert
        self.assertIn('welcome', response)

    @tornado.testing.gen_test
    def test_client_receives_rejoin_message(self):
        # act
        client = yield self.connect(LiveSyncSocketHandler.session_id, str(uuid.uuid4()))
        response = yield client.read_message()
        # assert
        self.assertIn('rejoin', response)

    @tornado.testing.gen_test
    def test_client_gets_registered_in_hub(self):
        # act
        client_id = uuid.uuid4()
        client = yield self.connect(client_id=client_id)
        # assert
        self.assertIn(str(client_id), Hub.clients())

    @tornado.testing.gen_test
    def test_client_gets_removed_from_hub(self):
        # act
        client_id = uuid.uuid4()
        client = yield self.connect(client_id=client_id)
        client.close()
        yield tornado.gen.sleep(0.1)
        # assert
        self.assertNotIn(str(client_id), Hub.clients())

    @tornado.testing.gen_test
    def test_handler_asks_hub_to_echo_messages(self):
        # act
        client_a = yield self.connect()
        client_b = yield self.connect()

        # discarding handshake
        yield client_b.read_message()

        client_a.write_message('{ "action": "test" }')
        message = yield client_b.read_message()

        # assert
        self.assertEqual('{ "action": "test" }', message)

    @tornado.testing.gen_test
    def test_redirect_action_updates_current_url(self):
        # act
        new_url = "http://url/"
        client = yield self.connect()
        client.write_message('{ "action": "redirect", "url": "%s"}' % new_url)

        yield tornado.gen.sleep(0.1)
        # assert
        self.assertEqual(LiveSyncSocketHandler.current_url, new_url)




from io import StringIO

from samon.elements import BaseElement
from samon.render import RenderedElement


class KrApp(BaseElement):
    def to_xml(self, io: StringIO, indent: int, rendered_element: RenderedElement):
        try:
            module = rendered_element.node_attributes.pop('module')
            entrypoint = rendered_element.node_attributes.pop('entrypoint')
            print('<script type="module">', file=io, end='')
            print(f"import('{module}').then((m) => {{ m['{entrypoint}'](m.html`", file=io, end='')
            rendered_element.to_js_template(io)
            print("`) });", file=io);
            print('</script>', file=io)
        except KeyError:  # just for backward compatibility
            with rendered_element.frame(io, indent):
                print('<script type="text/javascript">', file=io, end='')
                print(f'var ${self.xml_attrs["id"]}_data = `', file=io, end='')
                rendered_element.to_jsx(io)
                print('`;</script>', file=io)



from .document import Document
from .webcomponent import webcomponent


class BrythonLibMock:
    def __getattribute__(self, item):
        return self

    def __call__(self, *args, **kwargs):
        return self

    def __le__(self, other):
        pass

    def __ge__(self, other):
        pass


aio = BrythonLibMock()
ajax = BrythonLibMock()
html = BrythonLibMock()
local_storage = BrythonLibMock()
markdown = BrythonLibMock()
object_storage = BrythonLibMock()
session_storage = BrythonLibMock()
svg = BrythonLibMock()
template = BrythonLibMock()
timer = BrythonLibMock()
websocket = BrythonLibMock()
worker = BrythonLibMock()
widgets = BrythonLibMock()

window = BrythonLibMock()

document = Document()



import uuid
import json
from tornado.websocket import WebSocketHandler
from .hub import Hub


class LiveSyncSocketHandler(WebSocketHandler):
    current_url = None
    session_id = uuid.uuid4()

    def __init__(self, *args, **kwargs):
        super(LiveSyncSocketHandler, self).__init__(*args, **kwargs)
        self.id = None

    def check_origin(self, origin):
        return True

    @property
    def url(self):
        return self.request.headers.get("Origin", "") + '/'

    def open(self):
        self.id = self.get_query_argument('client_id', uuid.uuid4())
        Hub.register(self)

        if str(self.session_id) != self.get_query_argument('session_id', None):
            self.write_message(json.dumps({
                'action': 'welcome',
                'payload':  {
                    'client_id': str(self.id),
                    'session_id': str(self.session_id),
                    'current_url': self.current_url,
                },
            }))
        else:
            self.write_message(json.dumps({
                'action': 'rejoin',
                'payload':  {},
            }))

        if not LiveSyncSocketHandler.current_url:
            LiveSyncSocketHandler.current_url = self.url

    def on_connection_close(self):
        Hub.remove(self.id)

    def on_message(self, message):
        message_json = json.loads(message)
        action = message_json.get('action')

        if action == "redirect":
            LiveSyncSocketHandler.current_url = message_json.get('url')

        Hub.echo(self.id, message)



import re
import json
from django.conf import settings
from django.template.loader import render_to_string


class DjangoLiveSyncMiddleware(object):
    def __init__(self, get_response=None):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        return self.process_response(request, response)

    @staticmethod
    def process_response(request, response):
        if settings.DEBUG and 'text/html' in response['Content-Type'] and hasattr(response, 'content'):
            livesync_settings = getattr(settings, 'DJANGO_LIVESYNC')
            snippet = render_to_string('livesync/livesync_scripts.html', context={'LIVESYNC_SETTINGS': livesync_settings}).encode('utf-8')

            snippet += b'</body>'
            pattern = re.compile(b'</body>', re.IGNORECASE)
            response.content = pattern.sub(snippet, response.content)

        return response



from django.urls import path

from . import views

urlpatterns = [
    path('index/', views.index, name='index'),
    path('form/', views.FormView.as_view(), name='form'),
    path('modal/', views.modal, name='modal'),
    path('table/', views.table, name='table'),
    path('table/filtered/', views.FilteredTable.as_view(), name='filtered-table'),
    path('chart/', views.charts, name='charts'),
    path('card/', views.card, name='card'),
    path('kanban/', views.kanban, name='kanban'),
]

app_name = 'backend'



class Hub(object):
    @staticmethod
    def clients():
        if not hasattr(Hub, '_clients'):
            Hub._clients = dict()
        return Hub._clients

    @staticmethod
    def register(client):
        Hub.clients()[client.id] = client

    @staticmethod
    def remove(client_id):
        if client_id in Hub.clients():
            del Hub.clients()[client_id]

    @staticmethod
    def echo(sender_id, msg):
        for client in Hub.clients().values():
            if client.id != sender_id:
                client.write_message(msg)



import ast
import sys
from io import StringIO

from .elements import BaseElement
from .expressions import Bind


class Template:
    def __init__(self):
        self.root_element = None  # type: BaseElement
        self.source_path = None  # type: Path

    def _show_element_subtree(self, element: BaseElement, stdout, indent: int=1):
        spaces = ' ' * (indent - 1) * 4
        print(f'{spaces} {element.__class__.__name__} <{element.xml_tag}>', file=stdout)
        for child in element.children:
            self._show_element_subtree(child, stdout=stdout, indent=indent + 1)

    def show_element_tree(self, stdout=sys.stdout):
        return self._show_element_subtree(element=self.root_element, stdout=stdout)

    def _as_xml_subtree(self, element: BaseElement, io, newline: str, indent: int=1):
        spaces = ' ' * (indent - 1) * 4

        if not element.xml_tag:  # anonymus elements
            print(f'{spaces}{element.text}', file=io, end=newline)
        else:
            closing = '>' if len(element.children) else '/>'
            print(f'{spaces}<{element.xml_tag}{element.attrs_as_xml}{closing}', file=io, end=newline)
            for i, child in enumerate(element.children):
                self._as_xml_subtree(child, io, newline=newline, indent=indent + 1)

            if len(element.children):
                print(f'{spaces}</{element.xml_tag}>', file=io, end=newline)

    def _as_psx(self, element: BaseElement):
        args = [ast.Constant(value=element.xml_tag), ast.Dict(keys=[], values=[]), ast.List(elts=[])]

        values = []
        for v in element.xml_attrs.values():
            if isinstance(v, Bind):
                tree = ast.parse(v.expr)
                values.append(ast.Lambda(args=ast.arguments(args=[ast.arg(arg='self')], posonlyargs=[], defaults=[], kwarg=None, kw_defaults=[], kwonlyargs=[], vararg=None), body=tree.body[0].value))
            else:
                values.append(ast.Constant(v))

        args[1].keys = [ast.Constant(value=_) for _ in element.xml_attrs.keys()]
        args[1].values = values
        for child in element.children:
            if child.xml_tag is None:
                args[2].elts.append(ast.Constant(value=child.text))
            else:
                args[2].elts.append(self._as_psx(child))

        node = ast.Call(
            func=ast.Name(id='h'),
            args=args,
            keywords=[]
        )

        return node

    def serialize(self, output='xml'):
        if output == 'xml':
            io = StringIO()
            self.root_element.xml_attrs = {
                'xmlns:c': "https://doculabs.io/2020/xtmpl#control",
                'xmlns:b': "https://doculabs.io/2020/xtmpl#data-binding"
            }
            self._as_xml_subtree(element=self.root_element, io=io, newline='\n', indent=1)
            return io.getvalue()
        elif output == 'psx':
            return self._as_psx(element=self.root_element)
        else:
            raise NotImplementedError(f'Invalid output type: {output}')





class TemplateError(Exception):
    """Base class for all template errors."""
    pass


class TemplateNotFound(TemplateError):
    pass


class ElementNameConflict(Exception):
    def __init__(self, class_name: str):
        self.class_name = class_name

    def __str__(self):
        return 'Class conflict: {0}'.format(self.class_name)



import datetime

from django.core.files.uploadedfile import UploadedFile
from django.forms import widgets, fields
from django.templatetags.l10n import unlocalize

from samon.elements import BaseElement, AnonymusElement
from samon.constants import XML_NAMESPACE_DATA_BINDING
from samon.expressions import Bind
from samon.render import RenderedElement
from django_kirui.widgets import CheckboxSwitch


widgets.TextInput.input_type = 'kr-input'
widgets.NumberInput.input_type = 'kr-number-input'
widgets.Select.input_type = 'kr-select'
widgets.SelectMultiple.input_type = 'kr-multi-select-checkbox'
widgets.CheckboxSelectMultiple.input_type = 'kr-multi-select-checkbox'
widgets.CheckboxInput.input_type = 'kr-checkbox-switch'
widgets.DateInput.input_type = 'kr-date-picker'
widgets.Textarea.input_type = 'kr-textbox'


class RenderedField(RenderedElement):
    def _eval_node_attributes(self, context) -> dict:
        retval = super(RenderedField, self)._eval_node_attributes(context)
        self.bf: fields.BoundField = retval.pop('boundField')

        retval['value'] = self.bf.value()
        if retval['value'] is None:
            retval['value'] = ''

        # TODO: serializálásnál a datetime nem alakítható JSON-ná, ezt az átalakítást máshova kellene tenni
        if 'value' in retval.keys():
            if isinstance(retval['value'], (datetime.datetime, datetime.date)):
                retval['value'] = retval['value'].strftime('%Y-%m-%d')
            elif isinstance(retval['value'], UploadedFile):
                retval['value'] = ''

        if self.bf.name in self.bf.form.errors.keys():
            retval['error'] = list(self.bf.form.errors[self.bf.name])[0]

        retval['widget'] = self.bf.field.widget.input_type

        if self.bf.field.label:
            retval['label'] = self.bf.field.label

        retval['required'] = self.bf.field.required
        retval['field_id'] = f'id_{self.bf.name}'
        retval['name'] = self.bf.name
        retval['disabled'] = int(self.bf.field.disabled)
        # retval['onChange'] = f"{retval.pop('formReactContextRef')}.handleInputChange"

        return retval

    @property
    def children(self):
        if choices := getattr(self.bf.field, 'choices', None):
            value = self.node_attributes.get('value', None)
            if isinstance(value, (list, tuple)):
                value = [unlocalize(_) for _ in value]
            else:
                value = unlocalize(value)

            for choice in choices:
                if self.node_attributes['widget'] == 'kr-multi-select-checkbox':
                    el = BaseElement(xml_tag='kr-option-checkbox', xml_attrs={})
                else:
                    el = BaseElement(xml_tag='option', xml_attrs={})

                if hasattr(choice[0], 'value'):
                    el.xml_attrs['value'] = choice[0].value or ''
                else:
                    el.xml_attrs['value'] = choice[0] or ''

                if isinstance(value, list) and unlocalize(el.xml_attrs['value']) in value or value == unlocalize(el.xml_attrs['value']):
                    el.xml_attrs['selected'] = True

                el.children = [AnonymusElement(choice[1])]
                yield RenderedElement(el, context=self._context)

        for child in super().children:
            yield child


class KrField(BaseElement):
    RENDERED_ELEMENT_CLASS = RenderedField
    TAG_NAME = 'kr-form-field'


class RenderedForm(RenderedElement):
    @property
    def children(self):
        if self._element.context_var_name:
            form_obj = self._context[self._element.context_var_name]

            for name, field in form_obj.fields.items():
                bf = form_obj[name]
                xml_attrs = {
                    (None, 'name'): name,
                    (None, 'boundField'): bf,
                    # (None, 'formReactContextRef'): self._element.xml_attrs['reactContextRef'],
                    (None, 'field-width'): self._element.xml_attrs['field-width']
                }
                if bf.field.label:
                    xml_attrs[(None, 'label-width')] = self._element.xml_attrs['label-width']

                kr_field = KrField(xml_tag=KrField.TAG_NAME, xml_attrs=xml_attrs)
                yield RenderedField(kr_field, self._context)

        for child in super().children:
            yield child


class KrForm(BaseElement):
    RENDERED_ELEMENT_CLASS = RenderedForm

    def _parse_xml_attrs(self, xml_attrs):
        attrs = super()._parse_xml_attrs(xml_attrs)

        form_obj = attrs.pop(f'{{{XML_NAMESPACE_DATA_BINDING}}}object', None)
        if form_obj is None:
            var_name = None
        else:
            var_name = form_obj.expr

        if 'method' not in attrs.keys():
            attrs['method'] = 'POST'

        if f'{{{XML_NAMESPACE_DATA_BINDING}}}csrfmiddlewaretoken' not in attrs.keys():
            attrs[f'{{{XML_NAMESPACE_DATA_BINDING}}}csrfmiddlewaretoken'] = Bind(expr='djsamon.csrf_token')

        #if 'reactContextRef' not in attrs.keys():
        #    attrs['reactContextRef'] = var_name

        #if 'whenSubmit' not in attrs.keys():
        #    attrs['whenSubmit'] = f"{attrs['reactContextRef']}.handleSubmit"

        self.context_var_name = var_name
        return attrs
